VERSION 8000; ; 8300

* -----------------------------------------------------------------------------
* @filedoc
* @file Bwk_Gebaeude_Generieren.Csx
* @brief Gebude fr Topografielinien generieren
* 
* Dieses Beispielskript generiert Gebudewnde fr vorhandene Topografielinien.
* Fr die Generierung werden Topografielinien mit Kode 752 und 753 
* der Schicht "Bestand Gebude" ausgewertet.
* Eine korrekte Erzeugung der Wnde ist nur fr Linien mit Laufrichtung
* Uhrzeigersinn mglich. Im anderen Fall muss die Wanddicke negativ angesetzt werden.
*
* -----------------------------------------------------------------------------
* Copyright (C) IB&T
* 
* Gewhrleistung und Nutzung
* 
* Die Nutzung des Skriptes erfolgt stets auf eigene Gefahr. IB&T bernimmt keine
* Gewhr dafr, dass die Nutzung dieses Skriptes zu Ihrem gewnschten Ergebnis
* fhrt. Da CardScripte vorhandene Projektdaten durch Direktzugriff auf die
* jeweiligen Datenbanken auch ohne weitere Sicherheitsabfragen unbemerkt verndern
* oder lschen knnen, erfordert eine Nutzung entsprechende Kenntnisse und groe
* Sorgfalt. IB&T haftet nicht fr eventuell entstehende Datenverluste und
* Folgeschden, die durch die Nutzung von CardScripten entstehen.
*
* -----------------------------------------------------------------------------
* @author TB
* @endfiledoc
* -----------------------------------------------------------------------------
* @history
* -----------------------------------------------------------------------------

BILDSCHIRM GRAFIK                            | gibt die Meldungen im Arbeitsprotokoll aus

* --- Zuweisungen -------------------------------------------------------------

!SchichtName = "Bestand Gebude"             | Name der Schicht mit Topografielinien
Wh = 3,0                                     | Hhe der Gebudewand
Wd = 0,4                                     | Dicke der Gebudewand

* --- Deklarationen -----------------------------------------------------------

SYMBOLE /TOPOSYS/ TopoSys                    | Symbol fr das System der Topografiedaten
SYMBOLE /TOPOLINE/ Linie                     | Symbol fr eine Topografielinie
SYMBOLE /TOPOLINEINDEX/ XLinie               | Symbol fr das Linienindexobjekt

SYMBOLE /BUILDSYS/ BauwerkSys                | Symbol fr das Sytem der Bauwerke
SYMBOLE /BUILDSTRUCTURE/ Bauwerk             | Symbol fr das Bauwerk
SYMBOLE /BUILDASSEMBLY/ Baugruppe            | Symbol fr die Baugruppe
SYMBOLE /BUILDMODULE/ Bauteil                | Symbol fr das Bauteil 
SYMBOLE /BUILDBODY/ Koerper                  | Symbol fr den Baukrper
SYMBOLE /BUILDPOINTARRAY/ Array1; Array2     | Symbole Hilfspolygone

SYMBOLE /COORDINATE/ Ko1; Ko2                | Symbole fr Hilfspunkte
SYMBOLE /LINE/ HLinie; CLinie1; CLinie2      | Symbole fr Hilfslinien

* --- Ausfhrungen -----------------------------------------------------------

* Objekte fr die Datensysteme
TopoSys = Project.GetTopoSys()                     | liefert das Objekt fr das System der Topografierdaten 
BauwerkSys = Project.GetBuildSys()                 | liefert das Objekt fr das System der Bauwerke

* Linienindexobjekt
XLinie = TopoSys.GetLineIndex(!SchichtName)        | liefert alle Linien einer Schicht
XLinie.SetCodeRange(752; 753)                      | setzt den auszuwertenden Kodebereich

* ... Schleife ber kodierte Linien einer Schicht

* Linienobjekt
SOLANGE Linie = XLinie.GetNext()                   | liefert die nchste Linie

  GebNr = Linie.GetNumber()                        | liefert die Indexnummer der Linie 
  EleAnz = Linie.GetVertexCount()-1                | liefert die Anzahl der Elemente der Linie

  * 1. Ebene: Bauwerksobjekt ("Gebude")
  TEXTGEN !GebName; "Geb_#"; Gebnr                 | Name des Gebudes (aus Liniennummer)
  Bauwerk = BauwerkSys.GetStructure(!GebName)      | liefert ein vorhandenes Bauwerk
  WENN Bauwerk <> $NULL DANN                       | Wenn Bauwerk vorhanden ...
    Bauwerk.Delete()                               | lscht das Bauwerk mit allen Baugruppen
  WEITER
  Bauwerk = BauwerkSys.CreateStructure(!GebName)   | erzeugt das Bauwerk ("Gebude")

  * 2. Ebene: Baugruppenobjekt ("Gebudekorpus")
  Baugruppe = Bauwerk.CreateAssembly("Korpus")     | erzeugt Baugruppe fr Gebudekorpus (= alle Wnde) 

* ... Schleife ber alle Elemente der Linie

  SCHLEIFE i = 1; EleAnz

    * 3. Ebene: Bauteilobjekt ("Wand")
    TEXTGEN !Bauteil; "Wand_#"; i                  | Name des Bauteils (aus Elementnummer)
    Bauteil = Baugruppe.CreateModule(!Bauteil)     | erzeugt Bauteil fr Einzelwand
    Bauteil.SetMaterial("neutral_rot(#12)")        | setzt Material der Einzelwand

    * 4. Ebene: Baukrperobjekt (Teil der "Wand")
    Koerper = Bauteil.CreateBody()                 | erzeugt Baukrper des Bauteils
 
    * Hilfspunktobjekte "Ko1" und "Ko2": Koordinate und Hhe Wandanfang und -ende aus vorh. Linienelement
    Ko1 = Linie.GetCoordinate(i)                   | erzeugt Hilfspunkt Wandanfang
    Z1 = Ko1.GetZ()                                | liefert die Hhe
    Ko2 = Linie.GetCoordinate(i+1)                 | erzeugt Hilfspunkt Wandende
    Z2 = Ko2.GetZ()                                | liefert die Hhe
    Ko1.Draw(2; 9)                                 | visualisiert Koordinate Wandanfang

    * Hilfslinienobjekt "HLinie": Wandelement aus Wandanfang ("Ko1") und -ende ("Ko2")
    HLinie = NEWOBJECT /LINE/                      | erzeugt Hilfslinie: 
    HLinie.SetCoordinate(Ko1)                      |   Anfangspunkt des Elementes 
    HLinie.SetCoordinate2(Ko2)                     |   Endpunkt des Elementes
    EleLae = HLinie.GetDistance()                  | liefert die Wandlnge (fr Station Wandende)

    * Hilfslinienobjekte "CLinie1" und "CLinie2": senkrechte Linien am Wandanfang und -ende
    CLinie1 = HLinie.GetCrossLine(0)               | liefert senkrechte Linie am Anfang der Linie "HLinie" (= Wandanfang) 
    CLinie2 = HLinie.GetCrossLine(EleLae)          | liefert senkrechte Linie am Ende der Linie "HLinie" (= Wandende)

    * Hilfspolygonobjekt: Wandanfang
    Array1 = Koerper.CreatePointArray(1)           | erzeugt Anfangspolygon (1 = geschlossen):
    Array1.CreatePoint( Wd; 0;  0)                 |   erzeugt einen Punkt im Polygon
    Array1.CreatePoint( Wd; Wh; 0)                 |   - " -
    Array1.CreatePoint(  0; Wh; 0)                 |   - " -
    Array1.CreatePoint(  0; 0;  0)                 |   - " - (Polygon wird automatisch geschlossen)
    Array1.SetPositionLine(CLinie1; Z1)            |   platziert das Polygon mithilfe der Linie "CLinie1"         
    Array1.SetIncline(0,0; 1,0)                    |   stellt das Polygon auf (hier: 90 Grad senkrecht kippen)     
    Koerper.CreateCover(Array1; 0)                 |   Stirnflche der Wand wird geschlossen (Polygon wird gefllt)

    * Hilfspolygonobjekt: Wandende
    Array2 = Koerper.CreatePointArray(1)           | erzeugt Endpolygon (s.o.)
    Array2.CreatePoint( Wd; 0;  0)                 |   erzeugt einen Punkt im Polygon
    Array2.CreatePoint( Wd; Wh; 0)                 |   - " -
    Array2.CreatePoint(  0; Wh; 0)                 |   - " -
    Array2.CreatePoint(  0; 0;  0)                 |   - " - (Polygon wird automatisch geschlossen)
    Array2.SetPositionLine(CLinie2; Z2)            |   platziert das Polygon mithilfe der Linie "CLinie1"
    Array2.SetIncline(0,0; 1,0)                    |   stellt das Polygon auf (hier: 90 Grad senkrecht kippen)
    Koerper.CreateCover(Array2; 0)                 |   Stirnflche der Wand wird geschlossen (Polygon wird gefllt)
    * verbindet Anfangs- und Endpolygon            
    Koerper.CreateTube(Array1; Array2)             

* ... Ende der Schleife ber alle Elemente der Linie
  ENDE SCHLEIFE
  Baugruppe = $NULL

* ... Ende der Schleife ber alle Linien eines Kodes einer Schicht
ENDE SOLANGE

MELDUNG NEUTRAL "Gebudewnde wurden generiert"